pro convert_fmos_to_specpro, fmos1dfile, fmos2dfile, fmosnoisefile, name=name
  
;Wrapper to take FMOS spectra and convert them to individual files viewable with the SpecPro interface.
;fmos1dfile - fits file with an array holding the 1D spectra
;fmos2dfile - fits file with an array holding the 2D spectra
;fmosnoisefile - fits file with array holding the noise for the 1D spectra
;name - optional string giving the observation name, to be used in naming output 1d/2d files
;       If not specified, default to "fmos"
;
;Output:
;spec1d, spec2d files for each fiber.

if n_elements(name) eq 0 then begin
   name = 'fmos'
endif

fmos1d = mrdfits(fmos1dfile,0,head)
fmos2d = mrdfits(fmos2dfile,0,head)
fmosnoise = mrdfits(fmosnoisefile)

dim_fmos1d = size(fmos1d)
dim_fmos2d = size(fmos2d)

numfibers = dim_fmos1d[2]
num2dypix = dim_fmos2d[2] / numfibers
numxpix = dim_fmos1d[1]

;Create an array of structures to hold the extracted 1d, 2d
fmos_1dstruct = {flux:dblarr(numxpix), ivar:dblarr(numxpix), lambda:dblarr(numxpix)}
fmos_2dstruct = {flux:dblarr(numxpix, num2dypix), ivar:dblarr(numxpix, num2dypix), $
                 lambda:dblarr(numxpix, num2dypix)}

fmos1dstruct = replicate(fmos_1dstruct, numfibers)
fmos2dstruct = replicate(fmos_2dstruct, numfibers)

;Calculate the variance (suboptimally, this is a kluge)
;Also calculate median value at each wavelength position,
;subtract these values off later
ivar = dblarr(numxpix)
med2d = dblarr(numxpix)
med1d = dblarr(numxpix)
for j = 0,numxpix-1 do begin
   med1d[j] = median(fmos1d[j,*])
   med2d[j] = median(fmos2d[j,*])
endfor

med2dnew = dblarr(numxpix,num2dypix)
for j = 0, num2dypix-1 do begin
   med2dnew[*,j] = med2d
endfor

;Now populate the structures with the spectra
for i = 0, numfibers-1 do begin
   spec1d = fmos1d[*,i]
   spec2d = fmos2d[*,(num2dypix*i):(num2dypix*i)+num2dypix-1]
   fmos1dstruct[i].flux = spec1d - med1d 
   fmos2dstruct[i].flux = spec2d - med2dnew
   
   ;get ivar
   ivar = 1 / fmosnoise[*,i]^2
   for j = 0,numxpix-1 do begin
      fmos2dstruct[i].ivar[j,*] = ivar[j]
   endfor
   fmos1dstruct[i].ivar = ivar

   ;Populate the lambda field
   crval1=sxpar(head,'CRVAL1')
   cd = sxpar(head,'CD1_1')
   crpix1 = sxpar(head,'CRPIX1')
   xtemp = findgen(numxpix)
   lambda = crval1+cd*(xtemp+1-crpix1)
   
   fmos1dstruct[i].lambda = lambda
   for j = 0,numxpix-1 do begin
      fmos2dstruct[i].lambda[j,*] = lambda[j]
   endfor   

endfor

;Package the data in specpro format
for i = 0, numfibers-1 do begin
   slit = strcompress(string(i+1),/remove_all)
   if strlen(slit) eq 1 then begin
      slit = '00'+slit
   endif else if strlen(slit) eq 2 then begin
      slit = '0'+slit
   endif

   spec1dname = 'spec1d.'+name+'.'+slit+'.fiber.fits'
   spec2dname = 'spec2d.'+name+'.'+slit+'.fiber.fits'   
   
   ;Do sigma clipping to remove outliers in 1D
   meanflux  = mean(fmos1dstruct[i].flux)
   stddevflux = stddev(fmos1dstruct[i].flux) 
   medianivar  = median(fmos1dstruct[i].ivar)
   
   idxkeep = where((fmos1dstruct[i].flux gt meanflux-6*stddevflux) and (fmos1dstruct[i].flux lt meanflux+6*stddevflux) $
                  and (fmos1dstruct[i].ivar gt medianivar / 6.))
   fmos1dtosave = {flux:fmos1dstruct[i].flux[idxkeep], ivar:fmos1dstruct[i].ivar[idxkeep], lambda:fmos1dstruct[i].lambda[idxkeep]}
   
   mwrfits, fmos1dtosave, spec1dname, /create
   mwrfits, fmos2dstruct[i], spec2dname, /create

endfor

end ;convert_fmos_to_specpro
